@@ -19,17 +19,25 @@
 //! A new established [`Stream`] is in [`Direct`] (default) mode, where
 //! a `Read` or `Write` leads to **at most one** syscall, and
 //! an `Ok(0)` will be returned if frame head is not completely read or written.
-//! It can be converted to [`Guarded`] mode, which wraps `Read` or `Write` in a loop,
-//! where `Ok(0)` is handled internally.
+//! It can be converted to [`Guarded`] mode with [`Stream::guard`],
+//! which wraps `Read` or `Write` in a loop, where `Ok(0)` is handled internally.
 //!
 //! Stream itself does not buffer any payload data during
 //! a `Read` or `Write`, so there is no extra heap allocation.
 //!
-//! # Special attention
+//! # Masking payload
 //!
-//! Data written to stream are not automatically masked. A standard client should mask data
-//! before sending it. The mask key is prepared by a [`ClientRole`](crate::role::ClientRole),
-//! which can be set or fetched via [`Stream::set_write_mask_key`] and [`Stream::write_mask_key`].
+//! Data read from stream are automatically unmasked.
+//! However, data written to stream are **NOT** automatically masked,
+//! since a `Write` call requires an immutable `&[u8]`.
+//!
+//! A standard client(e.g. [`StandardClient`](crate::role::StandardClient))
+//! should mask the payload before sending it;
+//! A non-standard client (e.g. [`Client`](crate::role::Client)) which holds an empty mask key
+//! can simply skip this step.
+//!
+//! The mask key is prepared by [`ClientRole`](crate::role::ClientRole),
+//! which can be set or fetched via [`Stream::set_mask_key`] and [`Stream::mask_key`].
 //!
 //! Example:
 //!
@@ -49,13 +57,29 @@
 //!     apply_mask4(key, &mut buf);
 //!
 //!     // set mask key for next write
-//!     ws.set_write_mask_key(key)?;
+//!     ws.set_mask_key(key)?;
 //!
 //!     // write some data
 //!     ws.write_all(&buf)?;
 //!     Ok(())
 //! }
 //! ```
+//!
+//! # Automatic masking
+//!
+//! It is annoying to mask the payload each time before a write,
+//! and it will block us from using convenient functions like [`std::io::copy`].
+//!
+//! With `unsafe_auto_mask_write` fearure enabled, the provided immutable `&[u8]` will be casted
+//! to a mutable `&mut [u8]` then payload data can be automatically masked.
+//!
+//! This feature only has effects on [`AutoMaskClientRole`](crate::role::AutoMaskClientRole),
+//! where its inner mask key may be updated (depends on
+//! [`AutoMaskClientRole::UPDATE_MASK_KEY`](crate::role::AutoMaskClientRole::UPDATE_MASK_KEY))
+//! and used to mask the payload before each write.
+//! Other [`ClientRole`](crate::role::ClientRole) and [`ServerRole`](crate::role::ServerRole)
+//! are not affected. Related code lies in `src/stream/detail/write#L118`.
+//!
 
 mod read;
 mod write;
@@ -97,7 +121,6 @@ pub struct Stream<IO, Role, Guard = Direct> {
     read_state: ReadState,
     write_state: WriteState,
     heartbeat: HeartBeat,
-    _marker: PhantomData<Role>,
     __marker: PhantomData<Guard>,
 }
 
@@ -132,7 +155,6 @@ impl<IO, Role> Stream<IO, Role> {
             read_state: ReadState::new(),
             write_state: WriteState::new(),
             heartbeat: HeartBeat::new(),
-            _marker: PhantomData,
             __marker: PhantomData,
         }
     }
@@ -146,7 +168,6 @@ impl<IO, Role> Stream<IO, Role> {
             read_state: self.read_state,
             write_state: self.write_state,
             heartbeat: self.heartbeat,
-            _marker: PhantomData,
             __marker: PhantomData,
         }
     }
@@ -195,22 +216,32 @@ mod test {
         fn flush(&mut self) -> Result<()> { Ok(()) }
     }
 
-    pub fn make_frame<R: RoleHelper>(opcode: OpCode, len: usize) -> (Vec<u8>, Vec<u8>) {
-        let data: Vec<u8> = std::iter::repeat(rand::random::<u8>()).take(len).collect();
-        let mut data2 = data.clone();
-
+    pub fn make_head(opcode: OpCode, mask: Mask, len: usize) -> Vec<u8> {
         let mut tmp = vec![0; 14];
-        let head = FrameHead::new(
-            Fin::Y,
-            opcode,
-            R::write_mask_key(&R::new()),
-            PayloadLen::from_num(len as u64),
-        );
+        let head = FrameHead::new(Fin::Y, opcode, mask, PayloadLen::from_num(len as u64));
 
         let head_len = head.encode(&mut tmp).unwrap();
-        let mut frame = Vec::new();
-        let write_n = frame.write(&tmp[..head_len]).unwrap();
+        let mut head = Vec::new();
+        let write_n = head.write(&tmp[..head_len]).unwrap();
         assert_eq!(write_n, head_len);
+        head
+    }
+
+    pub fn make_data(len: usize) -> Vec<u8> {
+        std::iter::repeat(rand::random::<u8>()).take(len).collect()
+    }
+
+    pub fn make_frame<R: RoleHelper>(opcode: OpCode, len: usize) -> (Vec<u8>, Vec<u8>) {
+        make_frame_with_mask(opcode, R::new().mask_key(), len)
+    }
+
+    // data is unmasked
+    pub fn make_frame_with_mask(opcode: OpCode, mask: Mask, len: usize) -> (Vec<u8>, Vec<u8>) {
+        let data = make_data(len);
+        let mut data2 = data.clone();
+
+        let mut frame = make_head(opcode, mask, len);
+        let head_len = frame.len();
 
         frame.append(&mut data2);
         assert_eq!(frame.len(), len + head_len);
